@@ -59,8 +59,13 @@ function Mailbox._send_query(self, criteria, charset)
     _check_optional(criteria, { 'string', 'table' })
     _check_optional(charset, 'string')
 
+    local query
     if (criteria == nil) then
-        criteria = ''
+        query = 'ALL'
+    elseif (type(criteria) == 'string') then
+        query = 'ALL ' .. criteria
+    else 
+        query = _make_query(criteria)
     end
 
     if (charset == nil) then
@@ -76,13 +81,6 @@ function Mailbox._send_query(self, criteria, charset)
         return {}
     end
    
-    local query
-    if (type(criteria) == 'string') then
-        query = 'ALL ' .. criteria
-    else 
-        query = _make_query(criteria)
-    end
-
     local _, results = ifcore.search(self._account._imap, query, charset)
 
     if (type(options) == 'table' and options.close == true) then
