@@ -227,7 +227,7 @@ response_greeting(session *ssn)
 	if (check_bye(ibuf.data))
 		return -1;
 
-	verbose("S (%d): %s", ssn->socket, ibuf);
+	verbose("S (%d): %s", ssn->socket, ibuf.data);
 
 	if (xstrcasestr(ibuf.data, "* PREAUTH"))
 		return STATUS_RESPONSE_PREAUTH;
@@ -498,7 +498,8 @@ response_list(session *ssn, int tag, char **mboxs, char **folders)
 int
 response_search(session *ssn, int tag, char **mesgs)
 {
-	int r, min;
+	int r;
+	unsigned int min;
 	regexp *re;
 	char *b, *m;
 
@@ -516,8 +517,8 @@ response_search(session *ssn, int tag, char **mesgs)
 			*m = '\0';
 		}
 
-		min = re->pmatch[1].rm_eo - re->pmatch[1].rm_so < ibuf.len ?
-		    re->pmatch[1].rm_eo - re->pmatch[1].rm_so :
+		min = (unsigned int)(re->pmatch[1].rm_eo - re->pmatch[1].rm_so) < ibuf.len ?
+		    (unsigned int)(re->pmatch[1].rm_eo - re->pmatch[1].rm_so) :
 		    ibuf.len;
 
 		xstrncpy(m, b + re->pmatch[1].rm_so, min);
@@ -565,6 +566,100 @@ response_fetchfast(session *ssn, int tag, char **flags, char **date,
 		if (!regexec(re->preg, s, re->nmatch, re->pmatch, 0))
 			*size = xstrndup(s + re->pmatch[1].rm_so,
 			    re->pmatch[1].rm_eo - re->pmatch[1].rm_so);
+
+		xfree(s);
+	}
+
+	return r;
+}
+
+
+/*
+ * Process the data that server sent due to IMAP FETCH FLAGS client request.
+ */
+int
+response_fetchflags(session *ssn, int tag, char **flags)
+{
+	int r;
+	char *s;
+	regexp *re;
+
+	if ((r = response_generic(ssn, tag)) == -1)
+		return -1;
+
+	re = &responses[DATA_RESPONSE_FETCH];
+	if (!regexec(re->preg, ibuf.data, re->nmatch, re->pmatch, 0)) { 
+		s = xstrndup(ibuf.data + re->pmatch[1].rm_so,
+		    re->pmatch[1].rm_eo - re->pmatch[1].rm_so);
+
+		re = &responses[DATA_RESPONSE_FETCH_FLAGS];
+		if (!regexec(re->preg, s, re->nmatch, re->pmatch, 0))
+			*flags = xstrndup(s + re->pmatch[1].rm_so,
+			    re->pmatch[1].rm_eo - re->pmatch[1].rm_so);
+
+		xfree(s);
+	}
+
+	return r;
+}
+
+
+/*
+ * Process the data that server sent due to IMAP FETCH INTERNALDATE client
+ * request.
+ */
+int
+response_fetchdate(session *ssn, int tag, char **date)
+{
+	int r;
+	char *s;
+	regexp *re;
+
+	if ((r = response_generic(ssn, tag)) == -1)
+		return -1;
+
+	re = &responses[DATA_RESPONSE_FETCH];
+	if (!regexec(re->preg, ibuf.data, re->nmatch, re->pmatch, 0)) { 
+		s = xstrndup(ibuf.data + re->pmatch[1].rm_so,
+		    re->pmatch[1].rm_eo - re->pmatch[1].rm_so);
+
+		re = &responses[DATA_RESPONSE_FETCH_DATE];
+		if (!regexec(re->preg, s, re->nmatch, re->pmatch, 0))
+			*date = xstrndup(s + re->pmatch[1].rm_so,
+			    re->pmatch[1].rm_eo - re->pmatch[1].rm_so);
+
+		xfree(s);
+	}
+
+	return r;
+}
+
+
+/*
+ * Process the data that server sent due to IMAP FETCH RFC822.SIZE client
+ * request.
+ */
+int
+response_fetchsize(session *ssn, int tag, char **size)
+{
+	int r;
+	char *s;
+	regexp *re;
+
+	if ((r = response_generic(ssn, tag)) == -1)
+		return -1;
+
+	re = &responses[DATA_RESPONSE_FETCH];
+	if (!regexec(re->preg, ibuf.data, re->nmatch, re->pmatch, 0)) { 
+		s = xstrndup(ibuf.data + re->pmatch[1].rm_so,
+		    re->pmatch[1].rm_eo - re->pmatch[1].rm_so);
+
+		re = &responses[DATA_RESPONSE_FETCH_SIZE];
+		if (!regexec(re->preg, s, re->nmatch, re->pmatch, 0))
+			*size = xstrndup(s + re->pmatch[1].rm_so,
+			    re->pmatch[1].rm_eo - re->pmatch[1].rm_so);
+
+		xfree(s);
 	}
 
 	return r;
@@ -578,7 +673,8 @@ response_fetchfast(session *ssn, int tag, char **flags, char **date,
 int
 response_fetchbody(session *ssn, int tag, char **body, size_t *len)
 {
-	int r, match, offset;
+	int r, match;
+	unsigned int offset;
 	ssize_t n;
 	regexp *re;
 
