@@ -36,7 +36,7 @@ messages = myaccount.mymailbox:fetch_message(result)
 result = {}
 for msgid, msgtxt in pairs(messages) do
     if (pipe_to('bayesian-spam-filter', msgtxt) == 1) then
-        table.insert(result, msgid)
+        result[msgid] = true
     end
 end
 
